package org.supermind.crawl;

import org.apache.nutch.protocol.ProtocolOutput;
import org.supermind.crawl.util.LongPersister;

import java.io.IOException;
import java.net.URL;
import java.util.zip.Checksum;
import java.util.logging.Level;

/**
 * Default implementation of {@link FetchedURLs}. Saves URL checksums directly
 * to a MapFile.
 */
public class DefaultFetchedURLs implements FetchedURLs {
  private LongPersister persister;
  private Checksum checksum;

  public DefaultFetchedURLs() {
  }

  /**
   * Has this URL already been fetched?
   * @param url
   * @return
   */
  public boolean contains(URL url) {
    long chksum = getChecksum(url);
    boolean contains = false;
    try {
      contains = persister.contains(chksum);
    } catch (IOException e) {
      LOG.log(Level.WARNING, "Exception checking contains for " + url, e);
    }
//    Fetcher.LOG.finer("Contains " + url + " : " + contains);
    return contains;
  }

  public void insert(ScheduledURL url, ProtocolOutput output) throws IOException {
    long chksm = getChecksum(url.url);
    persister.add(chksm);
  }

  public ScheduledURL get(long id) {
    throw new UnsupportedOperationException("get() not supported");
  }

  public void init() throws IOException {
    persister.init();
  }

  public void close() throws IOException {
    persister.close();
  }

  public void setPersister(LongPersister persister) throws IOException {
    this.persister = persister;
  }

  public void setChecksum(Checksum checksum) {
    this.checksum = checksum;
  }

  /**
   * Create a 64-bit checksum by merging a 32-bit host checksum with the url's
   * 32-bit checksum. By using host checksum as sig. bits, urls can be easily
   * sorted by host.
   *
   * @param url
   * @return
   */
  protected long getChecksum(URL url) {
    String host = url.getHost();
    byte[] bytes = host.getBytes();
    checksum.update(bytes, 0, bytes.length);
    long hostchecksum = checksum.getValue() << 32;
    checksum.reset();

    bytes = url.toString().getBytes();
    checksum.update(bytes, 0, bytes.length);
    long urlchecksum = checksum.getValue();
    checksum.reset();

    return hostchecksum | urlchecksum;
  }
}
