package org.supermind.crawl;

import org.apache.nutch.io.Writable;

import java.net.URL;
import java.io.DataOutput;
import java.io.IOException;
import java.io.DataInput;

/**
 * A URL to be fetched. Roughly equivalent to Nutch's FetchListEntry.
 */
public class ScheduledURL implements Writable{
  // TODO: how to persist this??!
  static long currentItemId;

  /**
   * URL's id.
   */
  public long id;

  /**
   * Which seed this URL originated from.
   */
  public int seedIndex;

  /**
   * The immediate URL that contained this link.
   */
  public long parentId;

  public int retries;

  public long lastModified;

  /**
   * URL to fetch.
   */
  public URL url;

  static synchronized long assignId() {
    return currentItemId++;
  }

  public ScheduledURL() {
  }

  public ScheduledURL(int seedIndex, long parentId, URL url) {
    this.id = assignId();
    this.seedIndex = seedIndex;
    this.parentId = parentId;
    this.url = url;
  }

  public void write(DataOutput out) throws IOException {
    out.writeLong(id);
    out.writeInt(seedIndex);
    out.writeLong(parentId);
    out.writeInt(retries);
    out.writeLong(lastModified);
    out.writeUTF(url.toString());
  }

  public void readFields(DataInput in) throws IOException {
    id = in.readLong();
    seedIndex = in.readInt();
    parentId = in.readLong();
    retries = in.readInt();
    lastModified = in.readLong();
    url = new URL(in.readUTF());
  }

  public static ScheduledURL read(DataInput in) throws IOException {
    ScheduledURL scheduledURL = new ScheduledURL();
    scheduledURL.readFields(in);
    return scheduledURL;
  }

  public boolean equals(Object o) {
    if (this == o) return true;
    if (!(o instanceof ScheduledURL)) return false;

    final ScheduledURL scheduledURL = (ScheduledURL) o;

    if (id != scheduledURL.id) return false;
    if (lastModified != scheduledURL.lastModified) return false;
    if (parentId != scheduledURL.parentId) return false;
    if (retries != scheduledURL.retries) return false;
    if (seedIndex != scheduledURL.seedIndex) return false;
    if (!url.equals(scheduledURL.url)) return false;

    return true;
  }

  public int hashCode() {
    int result;
    result = (int) (id ^ (id >>> 32));
    result = 29 * result + seedIndex;
    result = 29 * result + (int) (parentId ^ (parentId >>> 32));
    result = 29 * result + retries;
    result = 29 * result + (int) (lastModified ^ (lastModified >>> 32));
    result = 29 * result + url.hashCode();
    return result;
  }

  public String toString() {
    return "ScheduledURL{" +
        "id=" + id +
        ", seedIndex=" + seedIndex +
        ", parentId=" + parentId +
        ", retries=" + retries +
        ", lastModified=" + lastModified +
        ", url=" + url +
        "}";
  }
}
