package org.supermind.crawl.util;

import org.apache.nutch.io.*;

import java.io.IOException;
import java.util.Comparator;

public class LongLongPersister extends MapFilePersister<Long, Long> {
  private LongWritable throwAwayValue = new LongWritable();
  private LongWritable throwAwayKey = new LongWritable();

  public boolean contains(long key) throws IOException {
    if (!buffer.containsKey(key)) {
      if (reader == null) reader = new MapFile.Reader(nfs, mapdir);
      return reader.seek(new LongWritable(key));
    } else {
      return true;
    }
  }

  /**
   * Get value for key.
   * @param key
   * @return value, or -1 if key doesn't exist
   * @throws IOException
   */
  public long getValue(long key) throws IOException {
    long result = -1;
    if (buffer.containsKey(key)) {
      result = buffer.get(key);
    }
    else {
      if (reader == null) reader = new MapFile.Reader(nfs, mapdir);
      LongWritable val = new LongWritable();
      val = (LongWritable) reader.get(new LongWritable(key), val);
      result = val != null ? val.get() : -1;
    }
    return result;
  }

  protected WritableComparator getKeyComparator() {
    return new LongWritable.Comparator();
  }

  public Comparator<Long> getTypeComparator() {
    return new Comparator<Long>() {
      public int compare(Long thisLong, Long thatLong) {
        long thisValue = thisLong.longValue();
        long thatValue = thatLong.longValue();
        return (thisValue<thatValue ? -1 : (thisValue==thatValue ? 0 : 1));
      }
    };
  }

  public WritableComparable getKeyInstance(Long aLong, boolean throwaway) {
    if(throwaway) {
      throwAwayKey.set(aLong);
      return throwAwayKey;
    } else {
      return aLong != null ? new LongWritable(aLong) : new LongWritable();
    }
  }

  public Writable getValueInstance(Long aLong, boolean throwaway) {
    if(throwaway) {
      throwAwayValue.set(aLong);
      return throwAwayValue;
    } else {
      return aLong != null ? new LongWritable(aLong) : new LongWritable();
    }
  }
}
