package org.supermind.crawl.util;

import org.apache.nutch.io.*;

import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Comparator;

/**
 * Saves Long values to a {@link MapFile}.
 */
public class LongPersister extends MapFilePersister<Long, NullWritable> {
  private LongWritable throwAwayValue = new LongWritable();

  public void add(long value) throws IOException {
    if(contains(value)) {
      LOG.warning("FOUND DUPLICATE");
    }
    super.add(value, null);
  }

  public boolean contains(long value) throws IOException {
    boolean b = buffer.containsKey(value);
    if (!b) {
      try {
        if (reader == null) reader = new MapFile.Reader(nfs, mapdir);
      } catch (FileNotFoundException e) {
        return false;
      }
      throwAwayValue.set(value);
      return reader.seek(throwAwayValue);
    } else {
      return true;
    }
  }

  protected WritableComparator getKeyComparator() {
    return new LongWritable.Comparator();
  }

  public Comparator<Long> getTypeComparator() {
    return new Comparator<Long>() {
      public int compare(Long thisLong, Long thatLong) {
        long thisValue = thisLong.longValue();
        long thatValue = thatLong.longValue();
        return (thisValue<thatValue ? -1 : (thisValue==thatValue ? 0 : 1));
      }
    };
  }

  public WritableComparable getKeyInstance(Long aLong, boolean throwaway) {
    if(throwaway) {
      throwAwayValue.set(aLong);
      return throwAwayValue;
    } else {
      return aLong != null ? new LongWritable(aLong) : new LongWritable();
    }
  }
}
