package org.supermind.crawl.util;

import junit.framework.TestCase;
import junit.textui.TestRunner;

import java.io.IOException;
import java.util.logging.Level;

import org.supermind.crawl.util.ChainedScatterTable;
import org.supermind.crawl.util.ScatterPurger;

public class TestChainedScatterTable extends TestCase {
  {
    ChainedScatterTable.LOG.setLevel(Level.FINER);
  }

  protected TestScatterPurger purger = new TestScatterPurger();
  ChainedScatterTable table = new ChainedScatterTable(purger);

  protected void setUp() throws Exception {
    super.setUp();
  }

  public void testHash() {
    long i = 0xffeeee;                // 16772846 = 00000000111111111110111011101110
    i *= 1000;                        // to get a long value = 1111100111101111010101000110110000
    assertEquals(3, table.hash(i));  // 3      = 00000000000000000000000000000011
  }

  public void testFind() {
    final int i = 500;
    int expected = table.insert(i);
    int result = table.find(i);
    assertEquals("Not finding proper bucket", expected, result);
  }

  public void testGetAddressToEvict() {
    fill();
    assertEquals(0, table.getAddressToEvict());
    assertEquals(1, table.getAddressToEvict());
    assertEquals(2, table.getAddressToEvict());

    for (int i = 0; i < table.array.length; ++i) {
      table.array[i].clockbit = true;
    }
    table.clockIndex = 0;

    assertEquals(0, table.getAddressToEvict());
    for (int i = 1; i < table.array.length; ++i) {
      assertEquals(0, table.array[i].clockbit);
    }

    final int v = table.insert(2000);
    table.clockIndex = v;
    table.find(2000);
    assertEquals(1, table.array[v].clockbit);
    assertEquals(v + 1, table.getAddressToEvict());
  }

  public void testEvict() {
//    fill();
//    assertEquals(0, table.insert(2000));
//    assertEquals(0, table.insert(2001));
//
//    table.clear();
    fillLongChain();
    System.out.println(table.array[0]);
    assertTrue(table.array[0].head);
    int i = 1000;
    table.insert(i);
    assertEquals(1, purger.idx);
    assertEquals(0, purger.value);
    assertEquals(i, table.array[purger.idx].value);

    i = 5000;
    table.insert(i);
    assertEquals(1, purger.idx);
    assertEquals(1000, purger.value);
    assertEquals(i, table.array[purger.idx].value);
  }

  public void testReconstitute() {
//    int expected = 2000;
//    int idx = table.insert(expected);
//    assertEquals(expected, table.reconstitute(idx));
//
//    fill();
    
  }

  private void fill() {
    for (int i = 0; i < table.array.length; ++i) {
      ChainedScatterTable.Entry e = table.array[i];
      e.next = i % table.array.length;
      e.value = (i + 1) * 0xff;
    }
    table.count = table.array.length;
  }

  private void fillLongChain() {
    for (int i = 0; i < table.array.length; ++i) {
      table.insert(i);
    }
  }

  public static void main(String[] args) {
    TestRunner.run(TestChainedScatterTable.class);
  }

  class TestScatterPurger implements ScatterPurger {
    int idx;
    long value;
  public void purge(ChainedScatterTable table, int idx, long value) throws IOException {
    this.idx = idx;
    this.value = value;
  }

  public void close() throws IOException {

  }

  public boolean contains(long v) throws IOException {
    return false;
  }
}
}

