package org.supermind.crawl.util;

import junit.framework.TestCase;
import org.apache.nutch.io.MD5Hash;
import org.supermind.crawl.util.LongPersister;
import org.supermind.crawl.util.LongLongPersister;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class TestMapFilePersister extends TestCase {
  File tmpfile;
  File baseDir;
  LongPersister lp;
  LongLongPersister llp;
  MD5Persister mdp;

  public TestMapFilePersister() {
    Logger.getLogger("org.supermind").setLevel(Level.FINE);
    Logger.getLogger("org.apache").setLevel(Level.FINER);
  }

  protected void setUp() throws Exception {
    baseDir = new File("c:/temp/mapdirtest");

    final org.apache.nutch.fs.NutchFileSystem nfs = new org.apache.nutch.fs.LocalFileSystem();
    lp = new LongPersister();
    lp.maxBufferSize = 10;
    lp.setOverwrite(true);
    lp.setNfs(nfs);
    lp.setBaseDir(baseDir.toString());
    lp.init();

    llp = new LongLongPersister();
    llp.maxBufferSize = 10;
    llp.setOverwrite(true);
    llp.setNfs(nfs);
    llp.setBaseDir(baseDir.toString());
    // don't init first

    mdp = new MD5Persister();
    mdp.maxBufferSize = 10;
    mdp.setOverwrite(true);
    mdp.setNfs(nfs);
    mdp.setBaseDir(baseDir.toString());

  }

  protected void tearDown() throws Exception {
    lp.close();
    llp.close();
  }

  public void testAdd() throws IOException {
    for (int i = 0; i < lp.maxBufferSize; ++i) {
      final long value = new Long(i);
      lp.add(value);
      assertTrue(lp.contains(value));
    }
    assertEquals(lp.maxBufferSize, lp.buffer.size());

    for (int i = 1; i < lp.maxBufferSize + 1; ++i) {
      final long value = new Long(i * lp.maxBufferSize);
      lp.add(value);
      assertTrue(lp.contains(value));
    }
    // buffer should have been flushed
    assertEquals(lp.maxBufferSize, lp.buffer.size());
    assertTrue(lp.contains(new Long(lp.maxBufferSize)));
    assertTrue(lp.contains(new Long(0)));
    lp.flushToDisk();
    assertEquals(0, lp.buffer.size());
    lp.close();

    // test LongLongPersister
    llp.init();
    for (int i = 0; i < llp.maxBufferSize; ++i) {
      llp.add((long)i, (long)i+i);
      assertTrue(llp.contains(new Long(i)));
      assertEquals(i + i, llp.getValue(new Long(i)));
    }
    llp.flushToDisk();
    for (int i = 0; i < llp.maxBufferSize; ++i) {
      assertEquals(i + i, llp.getValue(new Long(i)));
    }
    llp.close();

    // test MD5Persister
    mdp.init();
    for (int i = 0; i < mdp.maxBufferSize; ++i) {
      final MD5Hash hash = MD5Hash.digest(String.valueOf(i));
      mdp.add(hash);
      assertTrue(mdp.contains(hash));
    }
    mdp.flushToDisk();
    for (int i = 0; i < mdp.maxBufferSize; ++i) {
      assertTrue(mdp.contains(MD5Hash.digest(String.valueOf(i))));
    }
    mdp.close();

  }

  public void testBigAdd() throws IOException {
    lp.maxBufferSize = 5000;
    for (int i = 0; i < Integer.MAX_VALUE; ++i) {
      if(i%1000 ==0) System.out.println(i);
      final long value = new Long(i);
      lp.add(value);
      assertTrue(lp.contains(value));
    }
  }
}
